package tech.starwin.mvp;

import com.trello.rxlifecycle2.LifecycleProvider;
import com.trello.rxlifecycle2.android.ActivityEvent;


/**
 * Created by SiKang on 2018/9/14.
 */
public interface IView extends LifecycleProvider<ActivityEvent> {

    void onHttpStart(String action, boolean isShowLoading);

    void onHttpSuccess(String action, Object result);

    void onHttpError(String action, String msg);

    void onHttpFinish(String action);

}
